@@ -10,9 +10,9 @@
 #' is.biosoftwares.db.active('config.cfg')
 #' @export
 is.biosoftwares.db.active <- function(biosoftwares.db) {
-  biosoftwares.db <- normalizePath(biosoftwares.db)
-  identical(biosoftwares.db, Sys.getenv("BIO_SOFTWARES_DB_ACTIVE", system.file("extdata", "softwares_db_demo.yaml", 
-    package = "BioInstaller")))
+  biosoftwares.db <- normalizePath(biosoftwares.db, mustWork = FALSE)
+  identical(biosoftwares.db, Sys.getenv("BIO_SOFTWARES_DB_ACTIVE", system.file("extdata", 
+    "softwares_db_demo.yaml", package = "BioInstaller")))
 }
 
 #' Set BIO_SOFWARES_DB_ACTIVE as the BioInstaller db
